#define YY_REENTRANT 1
#define YY_TEXT_IS_ARRAY
#define YY_REENTRANT_BISON_PURE 1
#ifndef YY_REENTRANT
#define yytext sstext
#define yyleng ssleng
#define yyin ssin
#define yyout ssout
#endif
#define yy_create_buffer ss_create_buffer
#define yy_delete_buffer ss_delete_buffer
#define yy_scan_buffer ss_scan_buffer
#define yy_scan_string ss_scan_string
#define yy_scan_bytes ss_scan_bytes
#define yy_flex_debug ss_flex_debug
#define yy_init_buffer ss_init_buffer
#define yy_flush_buffer ss_flush_buffer
#define yy_load_buffer_state ss_load_buffer_state
#define yy_switch_to_buffer ss_switch_to_buffer
#define yylex sslex
#define yyrestart ssrestart
#define yylex_init sslex_init
#define yylex_destroy sslex_destroy
#define yyget_extra ssget_extra
#define yyset_extra ssset_extra
#define yyget_in ssget_in
#define yyset_in ssset_in
#define yyget_out ssget_out
#define yyset_out ssset_out
#define yyget_leng ssget_leng
#define yyget_text ssget_text
#define yyget_lineno ssget_lineno
#define yyset_lineno ssset_lineno
#ifdef YY_REENTRANT_BISON_PURE
#define yyget_lval ssget_lval
#define yyset_lval ssset_lval
#ifdef YYLTYPE
#define yyget_lloc ssget_lloc
#define yyset_lloc ssset_lloc
#endif
#endif
#define yywrap sswrap

/* -*-C-*- */
/* A lexical scanner generated by flex */

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5

/* %- */
/* begin standard C headers. */
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
/* end standard C headers. */
/* %+ */
/* %* */

#ifdef __cplusplus

/* %+ */
/* %* */

/* Use prototypes in function declarations. */
#define YY_USE_PROTOS

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else	/* ! __cplusplus */

#if __STDC__

#define YY_USE_PROTOS
#define YY_USE_CONST

#endif	/* __STDC__ */
#endif	/* ! __cplusplus */

#ifdef YY_USE_CONST
#define yyconst const
#else
#define yyconst
#endif


#ifdef YY_USE_PROTOS
#define YY_PROTO(proto) proto
#else
#define YY_PROTO(proto) ()
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)


#ifdef YY_REENTRANT

/* An opaque pointer. */
#ifndef YY_TYPEDEF_YY_SCANNER_T
#define YY_TYPEDEF_YY_SCANNER_T
typedef void* yyscan_t;
#endif

/* For use wherever a Global is accessed or assigned. */
#define YY_G(var) (((struct yy_globals_t*)yy_globals)->var)

/* For use in function prototypes to append the additional argument. */
#ifdef YY_USE_PROTOS
#define YY_LAST_ARG , yyscan_t yy_globals
#define YY_ONLY_ARG    yyscan_t yy_globals
#else
#define YY_LAST_ARG , yy_globals
#define YY_ONLY_ARG    yy_globals
#define YY_DECL_LAST_ARG yyscan_t yy_globals;
#endif

/* For use in function calls to pass the additional argument. */
#define YY_CALL_LAST_ARG  , yy_globals
#define YY_CALL_ONLY_ARG   yy_globals

/* For convenience, these vars (plus the bison vars far below)
   are macros in the reentrant scanner. */
#define yyin YY_G(yyin_r)
#define yyout YY_G(yyout_r)
#define yyextra YY_G(yyextra_r)
#define yyleng YY_G(yyleng_r)
#define yytext YY_G(yytext_r)
#define yylineno YY_G(yylineno_r)

int yylex_init YY_PROTO((yyscan_t* scanner));
int yylex_destroy YY_PROTO((yyscan_t scanner));

#else /* not YY_REENTRANT */

  /* Define these macros to be no-ops. */
#define YY_G(var) (var)
#define YY_LAST_ARG
#define YY_ONLY_ARG  void
#define YY_CALL_LAST_ARG
#define YY_CALL_ONLY_ARG
#define YY_DECL_LAST_ARG
#endif

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN YY_G(yy_start) = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START ((YY_G(yy_start) - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE yyrestart( yyin YY_CALL_LAST_ARG )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#define YY_BUF_SIZE 16384


#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

#ifndef YY_REENTRANT
extern size_t yyleng;
#endif

/* %- */
#ifndef YY_REENTRANT
extern FILE *yyin, *yyout;
#endif
/* %* */

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

/* The funky do-while in the following #define is used to turn the definition
 * int a single C statement (which needs a semi-colon terminator).  This
 * avoids problems with code like:
 *
 * 	if ( condition_holds )
 *		yyless( 5 );
 *	else
 *		do_something_else();
 *
 * Prior to using the do-while the compiler would get upset at the
 * "else" because it interpreted the "if" statement as being all
 * done when it reached the ';' after the yyless() call.
 */

/* Return all but the first 'n' matched characters back to the input stream. */

#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
		*yy_cp = YY_G(yy_hold_char); \
		YY_RESTORE_YY_MORE_OFFSET \
		YY_G(yy_c_buf_p) = yy_cp = yy_bp + n - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up yytext again */ \
		} \
	while ( 0 )

#define unput(c) yyunput( c, YY_G(yytext_ptr) YY_CALL_LAST_ARG )

/* The following is because we cannot portably get our hands on size_t
 * (without autoconf's help, which isn't available because we want
 * flex-generated scanners to compile on their own).
 */

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef unsigned int yy_size_t;
#endif

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct yy_buffer_state
	{
/* %- */
	FILE *yy_input_file;
/* %+ */
/* %* */

	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	yy_size_t yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	int yy_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int yy_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int yy_at_bol;

	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;
#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via yyrestart()), so that the user can continue scanning by
	 * just pointing yyin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2
	};
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* %- Standard (non-C++) definition */
/* %c */
#ifndef ssIN_HEADER
#ifndef YY_REENTRANT
static YY_BUFFER_STATE yy_current_buffer = 0;
#endif
/* %e */
#endif /* !ssIN_HEADER */
/* %* */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 */
#define YY_CURRENT_BUFFER yy_current_buffer


/* %- Standard (non-C++) definition */

#ifndef YY_REENTRANT
/* %c */
#ifndef ssIN_HEADER
/* yy_hold_char holds the character lost when yytext is formed. */
static char yy_hold_char;

static int yy_n_chars;		/* number of characters read into yy_ch_buf */


size_t yyleng;

/* Points to current character in buffer. */
static char *yy_c_buf_p = (char *) 0;
static int yy_init = 1;		/* whether we need to initialize */
static int yy_start = 0;	/* start state number */

/* Flag which is used to allow yywrap()'s to do buffer switches
 * instead of setting up a fresh yyin.  A bit of a hack ...
 */
static int yy_did_buffer_switch_on_eof;
/* %e */
#endif /* !ssIN_HEADER */
#endif /* end !YY_REENTRANT */

void yyrestart YY_PROTO(( FILE *input_file YY_LAST_ARG ));


void yy_switch_to_buffer YY_PROTO(( YY_BUFFER_STATE new_buffer YY_LAST_ARG ));
void yy_load_buffer_state YY_PROTO(( YY_ONLY_ARG ));
YY_BUFFER_STATE yy_create_buffer YY_PROTO(( FILE *file, int size YY_LAST_ARG ));
void yy_delete_buffer YY_PROTO(( YY_BUFFER_STATE b YY_LAST_ARG ));
void yy_init_buffer YY_PROTO(( YY_BUFFER_STATE b, FILE *file YY_LAST_ARG ));
void yy_flush_buffer YY_PROTO(( YY_BUFFER_STATE b YY_LAST_ARG ));

#define YY_FLUSH_BUFFER yy_flush_buffer( YY_G(yy_current_buffer) YY_CALL_LAST_ARG)

YY_BUFFER_STATE yy_scan_buffer YY_PROTO(( char *base, yy_size_t size YY_LAST_ARG ));
YY_BUFFER_STATE yy_scan_string YY_PROTO(( yyconst char *yy_str YY_LAST_ARG ));
YY_BUFFER_STATE yy_scan_bytes YY_PROTO(( yyconst char *bytes, int len YY_LAST_ARG ));

/* %* */

/* %c */
#ifndef ssIN_HEADER
static void *yy_flex_alloc YY_PROTO(( yy_size_t YY_LAST_ARG ));
static void *yy_flex_realloc YY_PROTO(( void *, yy_size_t YY_LAST_ARG ));
static void yy_flex_free YY_PROTO(( void * YY_LAST_ARG ));
/* %e */
#endif /* !ssIN_HEADER */

#define yy_new_buffer yy_create_buffer

#define yy_set_interactive(is_interactive) \
	{ \
	if ( ! YY_G(yy_current_buffer) ) \
		YY_G(yy_current_buffer) =    \
            yy_create_buffer( yyin, YY_BUF_SIZE YY_CALL_LAST_ARG); \
	YY_G(yy_current_buffer)->yy_is_interactive = is_interactive; \
	}

#define yy_set_bol(at_bol) \
	{ \
	if ( ! YY_G(yy_current_buffer) ) \
		YY_G(yy_current_buffer) =    \
            yy_create_buffer( yyin, YY_BUF_SIZE YY_CALL_LAST_ARG); \
	YY_G(yy_current_buffer)->yy_at_bol = at_bol; \
	}

#define YY_AT_BOL() (YY_G(yy_current_buffer)->yy_at_bol)

/* %% [1.0] yytext/yyin/yyout/yy_state_type/yylineno etc. def's & init go here */
/* Begin user sect3 */
#ifndef ssIN_HEADER
typedef unsigned char YY_CHAR;
#endif /* !ssIN_HEADER */
#ifndef ssIN_HEADER
#ifndef YY_REENTRANT
FILE *yyin = (FILE *) 0, *yyout = (FILE *) 0;
#endif
#endif /* !ssIN_HEADER */
#ifndef ssIN_HEADER
typedef int yy_state_type;
#endif /* !ssIN_HEADER */

/* %- Standard (non-C++) definition */
/* %c */
#ifndef ssIN_HEADER
static yy_state_type yy_get_previous_state YY_PROTO(( YY_ONLY_ARG ));
static yy_state_type yy_try_NUL_trans YY_PROTO(( yy_state_type current_state  YY_LAST_ARG));
static int yy_get_next_buffer YY_PROTO(( YY_ONLY_ARG ));
static void yy_fatal_error YY_PROTO(( yyconst char msg[] ));
/* %e */
#endif /* !ssIN_HEADER */
/* %* */

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up yytext.
 */
#define YY_DO_BEFORE_ACTION \
	YY_G(yytext_ptr) = yy_bp; \
/* %% [2.0] code to fiddle yytext and yyleng for yymore() goes here \ */\
	yyleng = (size_t) (yy_cp - yy_bp); \
	YY_G(yy_hold_char) = *yy_cp; \
	*yy_cp = '\0'; \
/* %% [3.0] code to copy yytext_ptr to yytext[] goes here, if %array \ */\
	if ( yyleng >= YYLMAX ) \
		YY_FATAL_ERROR( "token too large, exceeds YYLMAX" ); \
	yy_flex_strncpy( yytext, YY_G(yytext_ptr), yyleng + 1 YY_CALL_LAST_ARG); \
	YY_G(yy_c_buf_p) = yy_cp;

/* %* */

/* %c */
#ifndef ssIN_HEADER
/* %% [4.0] data tables for the DFA and the user's section 1 definitions go here */
#define YY_NUM_RULES 23
#define YY_END_OF_BUFFER 24
static yyconst short int yy_accept[56] =
    {   0,
        0,    0,    0,    0,    0,    0,   24,   22,   18,   18,
       22,   17,   21,    4,   19,    8,    5,    9,   21,   21,
       21,   21,   21,   21,   21,   18,    7,    0,   21,   10,
        6,   11,   21,   21,   14,   21,   21,   13,   21,   21,
       20,   12,   21,   15,   21,   21,   21,    2,   16,   21,
       21,   21,    3,    1,    0
    } ;

static yyconst int yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    1,    2,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    4,    5,    1,    1,    1,    1,    1,    6,    1,
        1,    7,    1,    8,    7,    7,    1,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    9,   10,   11,
       12,   13,    1,    7,   14,    7,   15,   16,   17,   18,
        7,   19,   20,    7,    7,   21,   22,   23,   24,    7,
        7,   25,   26,   27,   28,    7,   29,    7,    7,    7,
        1,    1,    1,    1,    1,    1,   14,    7,   15,   16,

       17,   18,    7,   19,   20,    7,    7,   21,   22,   23,
       24,    7,    7,   25,   26,   27,   28,    7,   29,    7,
        7,    7,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static yyconst int yy_meta[30] =
    {   0,
        1,    1,    1,    2,    1,    1,    3,    1,    2,    1,
        1,    1,    1,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3
    } ;

static yyconst short int yy_base[58] =
    {   0,
        0,    0,    0,    0,    0,    0,   68,   69,   28,   31,
       55,    0,    0,   69,   69,   54,   53,   52,   40,   37,
       35,   12,   35,   42,   39,   35,   69,   51,    0,   69,
       69,   69,   40,   31,    0,   27,   32,    0,   31,   34,
       69,    0,   28,    0,   28,   31,   22,    0,    0,   31,
       28,   17,    0,    0,   69,   39,   40
    } ;

static yyconst short int yy_def[58] =
    {   0,
       55,    1,    1,    1,    1,    1,   55,   55,   55,   55,
       55,   56,   57,   55,   55,   55,   55,   55,   57,   57,
       57,   57,   57,   57,   57,   55,   55,   56,   57,   55,
       55,   55,   57,   57,   57,   57,   57,   57,   57,   57,
       55,   57,   57,   57,   57,   57,   57,   57,   57,   57,
       57,   57,   57,   57,    0,   55,   55
    } ;

static yyconst short int yy_nxt[99] =
    {   0,
        8,    9,   10,    9,   11,   12,   13,   14,    8,   15,
       16,   17,   18,   19,   13,   13,   13,   20,   13,   21,
       13,   13,   22,   23,   13,   24,   13,   13,   25,   26,
       26,   26,   26,   26,   26,   36,   26,   26,   26,   37,
       28,   28,   29,   54,   53,   52,   51,   50,   49,   48,
       47,   46,   45,   44,   43,   42,   41,   40,   39,   38,
       35,   34,   33,   32,   31,   30,   27,   55,    7,   55,
       55,   55,   55,   55,   55,   55,   55,   55,   55,   55,
       55,   55,   55,   55,   55,   55,   55,   55,   55,   55,
       55,   55,   55,   55,   55,   55,   55,   55

    } ;

static yyconst short int yy_chk[99] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    9,
        9,    9,   10,   10,   10,   22,   26,   26,   26,   22,
       56,   56,   57,   52,   51,   50,   47,   46,   45,   43,
       40,   39,   37,   36,   34,   33,   28,   25,   24,   23,
       21,   20,   19,   18,   17,   16,   11,    7,   55,   55,
       55,   55,   55,   55,   55,   55,   55,   55,   55,   55,
       55,   55,   55,   55,   55,   55,   55,   55,   55,   55,
       55,   55,   55,   55,   55,   55,   55,   55

    } ;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
#ifndef YYLMAX
#define YYLMAX 8192
#endif

#ifndef YY_REENTRANT
char yytext[YYLMAX];
char *yytext_ptr;
#endif
#line 1 "icalsslexer.l"
#define INITIAL 0
#line 2 "icalsslexer.l"
/* -*- Mode: C -*-
  ======================================================================
  FILE: icalsslexer.l
  CREATOR: eric 8 Aug 2000
  
  DESCRIPTION:
  
  $Id: icalsslexer.c 2 2021-01-06 09:02:24Z erx $
  $Locker$

(C) COPYRIGHT 2000, Eric Busboom, http://www.softwarestudio.org

 This program is free software; you can redistribute it and/or modify
 it under the terms of either: 

    The LGPL as published by the Free Software Foundation, version
    2.1, available at: http://www.fsf.org/copyleft/lesser.html

  Or:

    The Mozilla Public License Version 1.0. You may obtain a copy of
    the License at http://www.mozilla.org/MPL/

 The Original Code is eric. The Initial Developer of the Original
 Code is Eric Busboom

  ======================================================================*/

#include "icalssyacc.h"
#include "icalgaugeimpl.h"
#include "assert.h"

#include <string.h> /* For strdup() */

#undef YYPURE
#define YYPURE

#undef SS_FATAL_ERROR
#define SS_FATAL_ERROR(msg) sserror(msg)

#define sql 1
#define string_value 2

#line 596 "lex.ss.c"
/* %e */
#endif /* !ssIN_HEADER */

/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
#ifndef YY_NO_UNISTD_H
/* %- */
#include <unistd.h>
/* %+ */
/* %* */
#endif /* !YY_NO_UNISTD_H */


#ifndef YY_EXTRA_TYPE
#define YY_EXTRA_TYPE void *
#endif

/* %- Reentrant structure and macros (non-C++). */
#ifdef YY_REENTRANT

/* %c */
#ifndef ssIN_HEADER
struct yy_globals_t
    {

    /* User-defined. Not touched by flex. */
    YY_EXTRA_TYPE yyextra_r;

    /* The rest are the same as the globals declared in the non-reentrant scanner. */
    FILE *yyin_r, *yyout_r;
    YY_BUFFER_STATE yy_current_buffer;
    char yy_hold_char;
    int yy_n_chars;
    int yyleng_r;
    char *yy_c_buf_p;
    int yy_init;
    int yy_start;
    int yy_did_buffer_switch_on_eof;
    int yy_start_stack_ptr;
    int yy_start_stack_depth;
    int *yy_start_stack;
    yy_state_type yy_last_accepting_state;
    char* yy_last_accepting_cpos;

    int yylineno_r;

#ifdef YY_TEXT_IS_ARRAY
    char yytext_r[YYLMAX];
    char *yytext_ptr;
    int yy_more_offset;
    int yy_prev_more_offset;
#else
    char *yytext_r;
    int yy_more_flag;
    int yy_more_len;
#endif

#ifdef YY_REENTRANT_BISON_PURE
    YYSTYPE * yylval_r;
#ifdef YYLTYPE
    YYLTYPE * yylloc_r;
#endif
#endif

    };
/* %e */
#endif /* !ssIN_HEADER */

/* %c */
#ifndef ssIN_HEADER
static int yy_init_globals YY_PROTO(( yyscan_t ));
/* %e */
#endif /* !ssIN_HEADER */

/* This must go here because YYSTYPE and YYLSTYPE are included
 * from bison output in section 1.*/
#ifdef YY_REENTRANT_BISON_PURE
#    define yylval YY_G(yylval_r)
#  ifdef YYLTYPE
#    define yylloc YY_G(yylloc_r)
#  endif
#endif /* YY_REENTRANT_BISON_PURE */

#endif /* end if YY_REENTRANT */

/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */
#ifndef YY_NO_GET_EXTRA
YY_EXTRA_TYPE yyget_extra YY_PROTO(( YY_ONLY_ARG ));
#endif

#ifndef YY_NO_SET_EXTRA
void yyset_extra YY_PROTO(( YY_EXTRA_TYPE user_defined YY_LAST_ARG ));
#endif

#ifndef YY_NO_GET_IN
FILE *yyget_in YY_PROTO(( YY_ONLY_ARG ));
#endif

#ifndef YY_NO_SET_IN
void yyset_in  YY_PROTO(( FILE * in_str YY_LAST_ARG ));
#endif

#ifndef YY_NO_GET_OUT
FILE *yyget_out YY_PROTO(( YY_ONLY_ARG ));
#endif

#ifndef YY_NO_SET_OUT
void yyset_out  YY_PROTO(( FILE * out_str YY_LAST_ARG ));
#endif

#ifndef YY_NO_GET_LENG
int yyget_leng YY_PROTO(( YY_ONLY_ARG ));
#endif

#ifndef YY_NO_GET_TEXT
char *yyget_text YY_PROTO(( YY_ONLY_ARG ));
#endif

#ifndef YY_NO_GET_LINENO
int yyget_lineno YY_PROTO(( YY_ONLY_ARG ));
#endif

#ifndef YY_NO_SET_LINENO
void yyset_lineno YY_PROTO(( int line_number YY_LAST_ARG ));
#endif

#ifdef YY_REENTRANT_BISON_PURE
#ifndef YY_NO_GET_LVAL
YYSTYPE * yyget_lval YY_PROTO(( YY_ONLY_ARG ));
#endif
void yyset_lval YY_PROTO(( YYSTYPE * yylvalp YY_LAST_ARG ));
#ifdef YYLTYPE
#ifndef YY_NO_GET_LLOC
   YYLTYPE *yyget_lloc YY_PROTO(( YY_ONLY_ARG ));
#endif
#ifndef YY_NO_SET_LLOC
    void yyset_lloc YY_PROTO(( YYLTYPE * yyllocp YY_LAST_ARG ));
#endif
#endif /* YYLTYPE */
#endif /* YY_REENTRANT_BISON_PURE */

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int yywrap YY_PROTO(( YY_ONLY_ARG ));
#else
extern int yywrap YY_PROTO(( YY_ONLY_ARG ));
#endif
#endif

/* %- */
/* %c */
#ifndef ssIN_HEADER
#ifndef YY_NO_UNPUT
static void yyunput YY_PROTO(( int c, char *buf_ptr  YY_LAST_ARG));
#endif
/* %e */
#endif /* !ssIN_HEADER */
/* %* */

#ifndef yytext_ptr
static void yy_flex_strncpy YY_PROTO(( char *, yyconst char *, int YY_LAST_ARG));
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen YY_PROTO(( yyconst char * YY_LAST_ARG));
#endif

#ifndef YY_NO_INPUT
/* %- Standard (non-C++) definition */
/* %c */
#ifndef ssIN_HEADER
#ifdef __cplusplus
static int yyinput YY_PROTO(( YY_ONLY_ARG ));
#else
static int input YY_PROTO(( YY_ONLY_ARG ));
#endif
/* %e */
#endif /* !ssIN_HEADER */
/* %* */
#endif

#if YY_STACK_USED
#ifndef YY_REENTRANT
/* %c */
#ifndef ssIN_HEADER
static int yy_start_stack_ptr = 0;
static int yy_start_stack_depth = 0;
static int *yy_start_stack = 0;
/* %e */
#endif /* !ssIN_HEADER */
#endif
#ifndef YY_NO_PUSH_STATE
static void yy_push_state YY_PROTO(( int new_state YY_LAST_ARG));
#endif
#ifndef YY_NO_POP_STATE
static void yy_pop_state YY_PROTO(( YY_ONLY_ARG ));
#endif
#ifndef YY_NO_TOP_STATE
static int yy_top_state YY_PROTO(( YY_ONLY_ARG ));
#endif

#else
#define YY_NO_PUSH_STATE 1
#define YY_NO_POP_STATE 1
#define YY_NO_TOP_STATE 1
#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#define YY_READ_BUF_SIZE 8192
#endif

/* Copy whatever the last rule matched to the standard output. */

#ifndef ECHO
/* %- Standard (non-C++) definition */
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO (void) fwrite( yytext, yyleng, 1, yyout )
/* %+ C++ definition */
/* %* */
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
/* %% [5.0] fread()/read() definition of YY_INPUT goes here unless we're doing C++ \ */\
	if ( YY_G(yy_current_buffer)->yy_is_interactive ) \
		{ \
		int c = '*'; \
		size_t n; \
		for ( n = 0; n < max_size && \
			     (c = getc( yyin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( yyin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else \
		{ \
		errno=0; \
		while ( (result = fread(buf, 1, max_size, yyin))==0 && ferror(yyin)) \
			{ \
			if( errno != EINTR) \
				{ \
				YY_FATAL_ERROR( "input in flex scanner failed" ); \
				break; \
				} \
			errno=0; \
			clearerr(yyin); \
			} \
		}
/* %+ C++ definition \ */\
/* %* */
#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
/* %- */
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg )
/* %+ */
/* %* */
#endif

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
/* %- Standard (non-C++) definition */

/* If the bison pure parser is used, then bison will provide
   one or two additional arguments. */

#ifdef YY_REENTRANT_BISON_PURE
#  ifdef YYLTYPE 
#    ifdef YY_USE_PROTOS
#        define YY_LEX_ARGS (YYSTYPE * yylvalp, YYLTYPE * yyllocp YY_LAST_ARG)
#    else
#        define YY_LEX_ARGS (yylvalp, yyllocp YY_LAST_ARG) \
                             YYSTYPE * yylvalp; YYLTYPE * yyllocp; YY_DECL_LAST_ARG
#    endif
#  else
#    ifdef YY_USE_PROTOS
#        define YY_LEX_ARGS (YYSTYPE * yylvalp YY_LAST_ARG)
#    else
#        define YY_LEX_ARGS (yylvalp YY_LAST_ARG) \
                             YYSTYPE * yylvalp; YY_DECL_LAST_ARG
#    endif
#  endif
#else
#  ifdef YY_USE_PROTOS
#      define YY_LEX_ARGS (YY_ONLY_ARG)
#  else
#      define YY_LEX_ARGS (YY_ONLY_ARG) YY_DECL_LAST_ARG
#  endif
#endif


extern int yylex YY_PROTO( YY_LEX_ARGS );

#define YY_DECL int yylex YY_LEX_ARGS
/* %+ C++ definition */
/* %* */
#endif

/* Code executed at the beginning of each rule, after yytext and yyleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

/* %% [6.0] YY_RULE_SETUP definition goes here */
#define YY_RULE_SETUP \
	YY_USER_ACTION

/* %c */
#ifndef ssIN_HEADER
YY_DECL
	{
	register yy_state_type yy_current_state;
	register char *yy_cp, *yy_bp;
	register int yy_act;

/* %% [7.0] user's declarations go here */
#line 66 "icalsslexer.l"






#line 959 "lex.ss.c"

#ifdef YY_REENTRANT_BISON_PURE
    yylval = yylvalp;
#ifdef YYLTYPE
    yylloc = yyllocp;
#endif
#endif

	if ( YY_G(yy_init) )
		{
		YY_G(yy_init) = 0;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

		if ( ! YY_G(yy_start) )
			YY_G(yy_start) = 1;	/* first start state */

		if ( ! yyin )
/* %- */
			yyin = stdin;
/* %+ */
/* %* */

		if ( ! yyout )
/* %- */
			yyout = stdout;
/* %+ */
/* %* */

		if ( ! YY_G(yy_current_buffer) )
			YY_G(yy_current_buffer) =
				yy_create_buffer( yyin, YY_BUF_SIZE YY_CALL_LAST_ARG);

		yy_load_buffer_state( YY_CALL_ONLY_ARG );
		}

	while ( 1 )		/* loops until end-of-file is reached */
		{
/* %% [8.0] yymore()-related code goes here */
		yy_cp = YY_G(yy_c_buf_p);

		/* Support of yytext. */
		*yy_cp = YY_G(yy_hold_char);

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

/* %% [9.0] code to set up and find next match goes here */
		yy_current_state = YY_G(yy_start);
yy_match:
		do
			{
			register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)];
			if ( yy_accept[yy_current_state] )
				{
				YY_G(yy_last_accepting_state) = yy_current_state;
				YY_G(yy_last_accepting_cpos) = yy_cp;
				}
			while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
				{
				yy_current_state = (int) yy_def[yy_current_state];
				if ( yy_current_state >= 56 )
					yy_c = yy_meta[(unsigned int) yy_c];
				}
			yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
			++yy_cp;
			}
		while ( yy_base[yy_current_state] != 69 );

yy_find_action:
/* %% [10.0] code to find the action number goes here */
		yy_act = yy_accept[yy_current_state];
		if ( yy_act == 0 )
			{ /* have to back up */
			yy_cp = YY_G(yy_last_accepting_cpos);
			yy_current_state = YY_G(yy_last_accepting_state);
			yy_act = yy_accept[yy_current_state];
			}

		YY_DO_BEFORE_ACTION;

/* %% [11.0] code for yylineno update goes here */

do_action:	/* This label is used only to access EOF actions. */

/* %% [12.0] debug code goes here */

		switch ( yy_act )
	{ /* beginning of action switch */
/* %% [13.0] actions go here */
			case 0: /* must back up */
			/* undo the effects of YY_DO_BEFORE_ACTION */
			*yy_cp = YY_G(yy_hold_char);
			yy_cp = YY_G(yy_last_accepting_cpos);
			yy_current_state = YY_G(yy_last_accepting_state);
			goto yy_find_action;

case 1:
YY_RULE_SETUP
#line 72 "icalsslexer.l"
{ return SELECT; }
	YY_BREAK
case 2:
YY_RULE_SETUP
#line 73 "icalsslexer.l"
{ return FROM; }
	YY_BREAK
case 3:
YY_RULE_SETUP
#line 74 "icalsslexer.l"
{ return WHERE; }
	YY_BREAK
case 4:
YY_RULE_SETUP
#line 75 "icalsslexer.l"
{ return COMMA; }
	YY_BREAK
case 5:
YY_RULE_SETUP
#line 76 "icalsslexer.l"
{ return EQUALS; }
	YY_BREAK
case 6:
YY_RULE_SETUP
#line 77 "icalsslexer.l"
{ return EQUALS; }
	YY_BREAK
case 7:
YY_RULE_SETUP
#line 78 "icalsslexer.l"
{ return NOTEQUALS; }
	YY_BREAK
case 8:
YY_RULE_SETUP
#line 79 "icalsslexer.l"
{ return LESS; }
	YY_BREAK
case 9:
YY_RULE_SETUP
#line 80 "icalsslexer.l"
{ return GREATER; }
	YY_BREAK
case 10:
YY_RULE_SETUP
#line 81 "icalsslexer.l"
{ return LESSEQUALS; }
	YY_BREAK
case 11:
YY_RULE_SETUP
#line 82 "icalsslexer.l"
{ return GREATEREQUALS; }
	YY_BREAK
case 12:
YY_RULE_SETUP
#line 83 "icalsslexer.l"
{ return AND; }
	YY_BREAK
case 13:
YY_RULE_SETUP
#line 84 "icalsslexer.l"
{ return OR; }
	YY_BREAK
case 14:
YY_RULE_SETUP
#line 85 "icalsslexer.l"
{ return IS; }
	YY_BREAK
case 15:
YY_RULE_SETUP
#line 86 "icalsslexer.l"
{ return NOT; }
	YY_BREAK
case 16:
YY_RULE_SETUP
#line 87 "icalsslexer.l"
{ return SQLNULL; }
	YY_BREAK
case 17:
YY_RULE_SETUP
#line 88 "icalsslexer.l"
{ return QUOTE; }
	YY_BREAK
case 18:
YY_RULE_SETUP
#line 89 "icalsslexer.l"
;			
	YY_BREAK
case 19:
YY_RULE_SETUP
#line 90 "icalsslexer.l"
{ return EOL; }
	YY_BREAK
case 20:
YY_RULE_SETUP
#line 92 "icalsslexer.l"
{
	int c = input(yy_globals);
	unput(c);
	if(c!='\''){
		yylvalp->v_string= icalmemory_tmp_copy(yytext);
		return STRING;
	} else {
		/*ssmore();*/
	}
}
	YY_BREAK
case 21:
YY_RULE_SETUP
#line 103 "icalsslexer.l"
{
        yylval->v_string= icalmemory_tmp_copy(yytext);
	return STRING; 
}
	YY_BREAK
case 22:
YY_RULE_SETUP
#line 109 "icalsslexer.l"
{ return yytext[0]; }
	YY_BREAK
case 23:
YY_RULE_SETUP
#line 111 "icalsslexer.l"
ECHO;
	YY_BREAK
#line 1188 "lex.ss.c"
case YY_STATE_EOF(INITIAL):
case YY_STATE_EOF(sql):
case YY_STATE_EOF(string_value):
	yyterminate();

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = (int) (yy_cp - YY_G(yytext_ptr)) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = YY_G(yy_hold_char);
		YY_RESTORE_YY_MORE_OFFSET

		if ( YY_G(yy_current_buffer)->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed yyin at a new source and called
			 * yylex().  If so, then we have to assure
			 * consistency between yy_current_buffer and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			YY_G(yy_n_chars) = YY_G(yy_current_buffer)->yy_n_chars;
			YY_G(yy_current_buffer)->yy_input_file = yyin;
			YY_G(yy_current_buffer)->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( YY_G(yy_c_buf_p) <= &YY_G(yy_current_buffer)->yy_ch_buf[YY_G(yy_n_chars)] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			YY_G(yy_c_buf_p) = YY_G(yytext_ptr) + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state( YY_CALL_ONLY_ARG );

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state YY_CALL_LAST_ARG);

			yy_bp = YY_G(yytext_ptr) + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++YY_G(yy_c_buf_p);
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
/* %% [14.0] code to do back-up for compressed tables and set up yy_cp goes here */
				yy_cp = YY_G(yy_c_buf_p);
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer( YY_CALL_ONLY_ARG ) )
			{
			case EOB_ACT_END_OF_FILE:
				{
				YY_G(yy_did_buffer_switch_on_eof) = 0;

				if ( yywrap( YY_CALL_ONLY_ARG ) )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * yytext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					YY_G(yy_c_buf_p) = YY_G(yytext_ptr) + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! YY_G(yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				YY_G(yy_c_buf_p) =
					YY_G(yytext_ptr) + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state( YY_CALL_ONLY_ARG );

				yy_cp = YY_G(yy_c_buf_p);
				yy_bp = YY_G(yytext_ptr) + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				YY_G(yy_c_buf_p) =
				&YY_G(yy_current_buffer)->yy_ch_buf[YY_G(yy_n_chars)];

				yy_current_state = yy_get_previous_state( YY_CALL_ONLY_ARG );

				yy_cp = YY_G(yy_c_buf_p);
				yy_bp = YY_G(yytext_ptr) + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
	} /* end of yylex */
/* %e */
#endif /* !ssIN_HEADER */
/* %+ */
/* %c */
#ifndef ssIN_HEADER
/* %e */
#endif /* !ssIN_HEADER */
/* %* */

/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */

/* %- */
/* %c */
#ifndef ssIN_HEADER
#ifdef YY_USE_PROTOS
static int yy_get_next_buffer(YY_ONLY_ARG)
#else
static int yy_get_next_buffer(YY_ONLY_ARG)
YY_DECL_LAST_ARG
#endif
/* %+ */
/* %* */
	{
	register char *dest = YY_G(yy_current_buffer)->yy_ch_buf;
	register char *source = YY_G(yytext_ptr);
	register int number_to_move, i;
	int ret_val;

	if ( YY_G(yy_c_buf_p) > &YY_G(yy_current_buffer)->yy_ch_buf[YY_G(yy_n_chars) + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( YY_G(yy_current_buffer)->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( YY_G(yy_c_buf_p) - YY_G(yytext_ptr) - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) (YY_G(yy_c_buf_p) - YY_G(yytext_ptr)) - 1;

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( YY_G(yy_current_buffer)->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		YY_G(yy_current_buffer)->yy_n_chars = YY_G(yy_n_chars) = 0;

	else
		{
			size_t num_to_read =
			YY_G(yy_current_buffer)->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */
#ifdef YY_USES_REJECT
			YY_FATAL_ERROR(
"input buffer overflow, can't enlarge buffer because scanner uses REJECT" );
#else

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = YY_G(yy_current_buffer);

			int yy_c_buf_p_offset =
				(int) (YY_G(yy_c_buf_p) - b->yy_ch_buf);

			if ( b->yy_is_our_buffer )
				{
				int new_size = b->yy_buf_size * 2;

				if ( new_size <= 0 )
					b->yy_buf_size += b->yy_buf_size / 8;
				else
					b->yy_buf_size *= 2;

				b->yy_ch_buf = (char *)
					/* Include room in for 2 EOB chars. */
					yy_flex_realloc( (void *) b->yy_ch_buf,
							 b->yy_buf_size + 2 YY_CALL_LAST_ARG );
				}
			else
				/* Can't grow it, we don't own it. */
				b->yy_ch_buf = 0;

			if ( ! b->yy_ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			YY_G(yy_c_buf_p) = &b->yy_ch_buf[yy_c_buf_p_offset];

			num_to_read = YY_G(yy_current_buffer)->yy_buf_size -
						number_to_move - 1;
#endif
			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&YY_G(yy_current_buffer)->yy_ch_buf[number_to_move]),
			YY_G(yy_n_chars), num_to_read );

		YY_G(yy_current_buffer)->yy_n_chars = YY_G(yy_n_chars);
		}

	if ( YY_G(yy_n_chars) == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			yyrestart( yyin  YY_CALL_LAST_ARG);
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			YY_G(yy_current_buffer)->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	YY_G(yy_n_chars) += number_to_move;
	YY_G(yy_current_buffer)->yy_ch_buf[YY_G(yy_n_chars)] = YY_END_OF_BUFFER_CHAR;
	YY_G(yy_current_buffer)->yy_ch_buf[YY_G(yy_n_chars) + 1] = YY_END_OF_BUFFER_CHAR;

	YY_G(yytext_ptr) = &YY_G(yy_current_buffer)->yy_ch_buf[0];

	return ret_val;
	}
/* %e */
#endif /* !ssIN_HEADER */

/* yy_get_previous_state - get the state just before the EOB char was reached */

/* %- */
/* %c */
#ifndef ssIN_HEADER
#ifdef YY_USE_PROTOS
static yy_state_type yy_get_previous_state(YY_ONLY_ARG)
#else
static yy_state_type yy_get_previous_state(YY_ONLY_ARG)
YY_DECL_LAST_ARG
#endif
/* %+ */
/* %* */
	{
	register yy_state_type yy_current_state;
	register char *yy_cp;

/* %% [15.0] code to get the start state into yy_current_state goes here */
	yy_current_state = YY_G(yy_start);

	for ( yy_cp = YY_G(yytext_ptr) + YY_MORE_ADJ; yy_cp < YY_G(yy_c_buf_p); ++yy_cp )
		{
/* %% [16.0] code to find the next state goes here */
		register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
		if ( yy_accept[yy_current_state] )
			{
			YY_G(yy_last_accepting_state) = yy_current_state;
			YY_G(yy_last_accepting_cpos) = yy_cp;
			}
		while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
			{
			yy_current_state = (int) yy_def[yy_current_state];
			if ( yy_current_state >= 56 )
				yy_c = yy_meta[(unsigned int) yy_c];
			}
		yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
		}

	return yy_current_state;
	}


/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */

/* %- */
#ifdef YY_USE_PROTOS
static yy_state_type yy_try_NUL_trans( yy_state_type yy_current_state YY_LAST_ARG )
#else
static yy_state_type yy_try_NUL_trans( yy_current_state YY_LAST_ARG )
yy_state_type yy_current_state;
YY_DECL_LAST_ARG
#endif
/* %+ */
/* %* */
	{
	register int yy_is_jam;
/* %% [17.0] code to find the next state, and perhaps do backing up, goes here */
	register char *yy_cp = YY_G(yy_c_buf_p);

	register YY_CHAR yy_c = 1;
	if ( yy_accept[yy_current_state] )
		{
		YY_G(yy_last_accepting_state) = yy_current_state;
		YY_G(yy_last_accepting_cpos) = yy_cp;
		}
	while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		{
		yy_current_state = (int) yy_def[yy_current_state];
		if ( yy_current_state >= 56 )
			yy_c = yy_meta[(unsigned int) yy_c];
		}
	yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
	yy_is_jam = (yy_current_state == 55);

	return yy_is_jam ? 0 : yy_current_state;
	}


/* %- */
#ifndef YY_NO_UNPUT
#ifdef YY_USE_PROTOS
static void yyunput( int c, register char *yy_bp YY_LAST_ARG )
#else
static void yyunput( c, yy_bp YY_LAST_ARG)
int c;
register char *yy_bp;
YY_DECL_LAST_ARG
#endif
/* %+ */
/* %* */
	{
	register char *yy_cp = YY_G(yy_c_buf_p);

	/* undo effects of setting up yytext */
	*yy_cp = YY_G(yy_hold_char);

	if ( yy_cp < YY_G(yy_current_buffer)->yy_ch_buf + 2 )
		{ /* need to shift things up to make room */
		/* +2 for EOB chars. */
		register int number_to_move = YY_G(yy_n_chars) + 2;
		register char *dest = &YY_G(yy_current_buffer)->yy_ch_buf[
					YY_G(yy_current_buffer)->yy_buf_size + 2];
		register char *source =
				&YY_G(yy_current_buffer)->yy_ch_buf[number_to_move];

		while ( source > YY_G(yy_current_buffer)->yy_ch_buf )
			*--dest = *--source;

		yy_cp += (int) (dest - source);
		yy_bp += (int) (dest - source);
		YY_G(yy_current_buffer)->yy_n_chars =
			YY_G(yy_n_chars) = YY_G(yy_current_buffer)->yy_buf_size;

		if ( yy_cp < YY_G(yy_current_buffer)->yy_ch_buf + 2 )
			YY_FATAL_ERROR( "flex scanner push-back overflow" );
		}

	*--yy_cp = (char) c;

/* %% [18.0] update yylineno here */

	YY_G(yytext_ptr) = yy_bp;
	YY_G(yy_hold_char) = *yy_cp;
	YY_G(yy_c_buf_p) = yy_cp;
	}
/* %- */
#endif	/* ifndef YY_NO_UNPUT */
/* %* */


/* %- */
#ifndef YY_NO_INPUT
#ifdef __cplusplus
static int yyinput(YY_ONLY_ARG)
#else
#ifdef YY_USE_PROTOS
static int input(YY_ONLY_ARG)
#else
static int input(YY_ONLY_ARG)
    YY_DECL_LAST_ARG
#endif
#endif
/* %+ */
/* %* */
	{
	int c;

	*YY_G(yy_c_buf_p) = YY_G(yy_hold_char);

	if ( *YY_G(yy_c_buf_p) == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( YY_G(yy_c_buf_p) < &YY_G(yy_current_buffer)->yy_ch_buf[YY_G(yy_n_chars)] )
			/* This was really a NUL. */
			*YY_G(yy_c_buf_p) = '\0';

		else
			{ /* need more input */
			int offset = YY_G(yy_c_buf_p) - YY_G(yytext_ptr);
			++YY_G(yy_c_buf_p);

			switch ( yy_get_next_buffer( YY_CALL_ONLY_ARG ) )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because yy_g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					yyrestart( yyin YY_CALL_LAST_ARG);

					/* fall through */

				case EOB_ACT_END_OF_FILE:
					{
					if ( yywrap( YY_CALL_ONLY_ARG ) )
						return EOF;

					if ( ! YY_G(yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
#ifdef __cplusplus
					return yyinput(YY_CALL_ONLY_ARG);
#else
					return input(YY_CALL_ONLY_ARG);
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					YY_G(yy_c_buf_p) = YY_G(yytext_ptr) + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) YY_G(yy_c_buf_p);	/* cast for 8-bit char's */
	*YY_G(yy_c_buf_p) = '\0';	/* preserve yytext */
	YY_G(yy_hold_char) = *++YY_G(yy_c_buf_p);

/* %% [19.0] update BOL and yylineno */

	return c;
	}
/* %- */
#endif	/* ifndef YY_NO_INPUT */
/* %* */

/* %- */
#ifdef YY_USE_PROTOS
void yyrestart( FILE *input_file YY_LAST_ARG)
#else
void yyrestart( input_file YY_LAST_ARG)
FILE *input_file;
YY_DECL_LAST_ARG
#endif
/* %+ */
/* %* */
	{
	if ( ! YY_G(yy_current_buffer) )
		YY_G(yy_current_buffer) =
            yy_create_buffer( yyin, YY_BUF_SIZE YY_CALL_LAST_ARG);

	yy_init_buffer( YY_G(yy_current_buffer), input_file YY_CALL_LAST_ARG);
	yy_load_buffer_state( YY_CALL_ONLY_ARG );
	}


/* %- */
#ifdef YY_USE_PROTOS
void yy_switch_to_buffer( YY_BUFFER_STATE new_buffer YY_LAST_ARG )
#else
void yy_switch_to_buffer( new_buffer YY_LAST_ARG )
YY_BUFFER_STATE new_buffer;
YY_DECL_LAST_ARG
#endif
/* %+ */
/* %* */
	{
	if ( YY_G(yy_current_buffer) == new_buffer )
		return;

	if ( YY_G(yy_current_buffer) )
		{
		/* Flush out information for old buffer. */
		*YY_G(yy_c_buf_p) = YY_G(yy_hold_char);
		YY_G(yy_current_buffer)->yy_buf_pos = YY_G(yy_c_buf_p);
		YY_G(yy_current_buffer)->yy_n_chars = YY_G(yy_n_chars);
		}

	YY_G(yy_current_buffer) = new_buffer;
	yy_load_buffer_state( YY_CALL_ONLY_ARG );

	/* We don't actually know whether we did this switch during
	 * EOF (yywrap()) processing, but the only time this flag
	 * is looked at is after yywrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	YY_G(yy_did_buffer_switch_on_eof) = 1;
	}


/* %- */
#ifdef YY_USE_PROTOS
void yy_load_buffer_state( YY_ONLY_ARG )
#else
void yy_load_buffer_state(YY_ONLY_ARG )
YY_DECL_LAST_ARG
#endif
/* %+ */
/* %* */
	{
	YY_G(yy_n_chars) = YY_G(yy_current_buffer)->yy_n_chars;
	YY_G(yytext_ptr) = YY_G(yy_c_buf_p) = YY_G(yy_current_buffer)->yy_buf_pos;
	yyin = YY_G(yy_current_buffer)->yy_input_file;
	YY_G(yy_hold_char) = *YY_G(yy_c_buf_p);
	}


/* %- */
#ifdef YY_USE_PROTOS
YY_BUFFER_STATE yy_create_buffer( FILE *file, int size YY_LAST_ARG)
#else
YY_BUFFER_STATE yy_create_buffer( file, size YY_LAST_ARG)
FILE *file;
int size;
YY_DECL_LAST_ARG
#endif
/* %+ */
/* %* */
	{
	YY_BUFFER_STATE b;

	b = (YY_BUFFER_STATE) yy_flex_alloc( sizeof( struct yy_buffer_state ) YY_CALL_LAST_ARG );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) yy_flex_alloc( b->yy_buf_size + 2 YY_CALL_LAST_ARG );
	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_is_our_buffer = 1;

	yy_init_buffer( b, file YY_CALL_LAST_ARG);

	return b;
	}


/* %- */
#ifdef YY_USE_PROTOS
void yy_delete_buffer( YY_BUFFER_STATE b YY_LAST_ARG)
#else
void yy_delete_buffer( b YY_LAST_ARG)
YY_BUFFER_STATE b;
YY_DECL_LAST_ARG
#endif
/* %+ */
/* %* */
	{
	if ( ! b )
		return;

	if ( b == YY_G(yy_current_buffer) )
		YY_G(yy_current_buffer) = (YY_BUFFER_STATE) 0;

	if ( b->yy_is_our_buffer )
		yy_flex_free( (void *) b->yy_ch_buf YY_CALL_LAST_ARG );

	yy_flex_free( (void *) b YY_CALL_LAST_ARG );
	}


/* %- */
#ifndef YY_ALWAYS_INTERACTIVE
#ifndef YY_NEVER_INTERACTIVE
#ifdef __cplusplus
extern "C" int isatty YY_PROTO(( int ));
#else
extern int isatty YY_PROTO(( int ));
#endif /* __cplusplus */
#endif /* !YY_NEVER_INTERACTIVE */
#endif /* !YY_ALWAYS_INTERACTIVE */

#ifdef YY_USE_PROTOS
void yy_init_buffer( YY_BUFFER_STATE b, FILE *file YY_LAST_ARG)
#else
void yy_init_buffer( b, file YY_LAST_ARG)
YY_BUFFER_STATE b;
FILE *file;
YY_DECL_LAST_ARG
#endif

/* %+ */
/* %* */

	{
	int oerrno = errno;

	yy_flush_buffer( b YY_CALL_LAST_ARG);

	b->yy_input_file = file;
	b->yy_fill_buffer = 1;

/* %- */
#if YY_ALWAYS_INTERACTIVE
	b->yy_is_interactive = 1;
#else
#if YY_NEVER_INTERACTIVE
	b->yy_is_interactive = 0;
#else
	b->yy_is_interactive = file ? (isatty( fileno(file) ) > 0) : 0;
#endif
#endif
/* %+ */
/* %* */
	errno = oerrno;
	}


/* %- */
#ifdef YY_USE_PROTOS
void yy_flush_buffer( YY_BUFFER_STATE b YY_LAST_ARG )
#else
void yy_flush_buffer( b YY_LAST_ARG )
YY_BUFFER_STATE b;
YY_DECL_LAST_ARG
#endif

/* %+ */
/* %* */
	{
	if ( ! b )
		return;

	b->yy_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[0];

	b->yy_at_bol = 1;
	b->yy_buffer_status = YY_BUFFER_NEW;

	if ( b == YY_G(yy_current_buffer) )
		yy_load_buffer_state( YY_CALL_ONLY_ARG );
	}
/* %* */


#ifndef YY_NO_SCAN_BUFFER
/* %- */
#ifdef YY_USE_PROTOS
YY_BUFFER_STATE yy_scan_buffer( char *base, yy_size_t size YY_LAST_ARG )
#else
YY_BUFFER_STATE yy_scan_buffer( base, size YY_LAST_ARG )
char *base;
yy_size_t size;
YY_DECL_LAST_ARG
#endif
	{
	YY_BUFFER_STATE b;

	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return 0;

	b = (YY_BUFFER_STATE) yy_flex_alloc( sizeof( struct yy_buffer_state ) YY_CALL_LAST_ARG );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_scan_buffer()" );

	b->yy_buf_size = size - 2;	/* "- 2" to take care of EOB's */
	b->yy_buf_pos = b->yy_ch_buf = base;
	b->yy_is_our_buffer = 0;
	b->yy_input_file = 0;
	b->yy_n_chars = b->yy_buf_size;
	b->yy_is_interactive = 0;
	b->yy_at_bol = 1;
	b->yy_fill_buffer = 0;
	b->yy_buffer_status = YY_BUFFER_NEW;

	yy_switch_to_buffer( b YY_CALL_LAST_ARG );

	return b;
	}
/* %* */
#endif


#ifndef YY_NO_SCAN_STRING
/* %- */
#ifdef YY_USE_PROTOS
YY_BUFFER_STATE yy_scan_string( yyconst char *yy_str YY_LAST_ARG )
#else
YY_BUFFER_STATE yy_scan_string( yy_str YY_LAST_ARG)
yyconst char *yy_str;
YY_DECL_LAST_ARG
#endif
	{
	int len;
	for ( len = 0; yy_str[len]; ++len )
		;

	return yy_scan_bytes( yy_str, len YY_CALL_LAST_ARG);
	}
/* %* */
#endif


#ifndef YY_NO_SCAN_BYTES
/* %- */
#ifdef YY_USE_PROTOS
YY_BUFFER_STATE yy_scan_bytes( yyconst char *bytes, int len YY_LAST_ARG)
#else
YY_BUFFER_STATE yy_scan_bytes( bytes, len YY_LAST_ARG)
yyconst char *bytes;
YY_DECL_LAST_ARG
int len;
#endif
	{
	YY_BUFFER_STATE b;
	char *buf;
	yy_size_t n;
	int i;

	/* Get memory for full buffer, including space for trailing EOB's. */
	n = len + 2;
	buf = (char *) yy_flex_alloc( n YY_CALL_LAST_ARG );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_scan_bytes()" );

	for ( i = 0; i < len; ++i )
		buf[i] = bytes[i];

	buf[len] = buf[len+1] = YY_END_OF_BUFFER_CHAR;

	b = yy_scan_buffer( buf, n YY_CALL_LAST_ARG);
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in yy_scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->yy_is_our_buffer = 1;

	return b;
	}
/* %* */
#endif


#ifndef YY_NO_PUSH_STATE
/* %- */
#ifdef YY_USE_PROTOS
static void yy_push_state( int new_state YY_LAST_ARG)
#else
static void yy_push_state( new_state YY_LAST_ARG)
int new_state;
YY_DECL_LAST_ARG
#endif
/* %+ */
/* %* */
	{
	if ( YY_G(yy_start_stack_ptr) >= YY_G(yy_start_stack_depth) )
		{
		yy_size_t new_size;

		YY_G(yy_start_stack_depth) += YY_START_STACK_INCR;
		new_size = YY_G(yy_start_stack_depth) * sizeof( int );

		if ( ! YY_G(yy_start_stack) )
			YY_G(yy_start_stack) = (int *) yy_flex_alloc( new_size YY_CALL_LAST_ARG );

		else
			YY_G(yy_start_stack) = (int *) yy_flex_realloc(
					(void *) YY_G(yy_start_stack), new_size YY_CALL_LAST_ARG );

		if ( ! YY_G(yy_start_stack) )
			YY_FATAL_ERROR(
			"out of memory expanding start-condition stack" );
		}

	YY_G(yy_start_stack)[YY_G(yy_start_stack_ptr)++] = YY_START;

	BEGIN(new_state);
	}
#endif


#ifndef YY_NO_POP_STATE
/* %- */
#ifdef YY_USE_PROTOS
static void yy_pop_state( YY_ONLY_ARG )
#else
static void yy_pop_state( YY_ONLY_ARG )
YY_DECL_LAST_ARG
#endif
/* %+ */
/* %* */
	{
	if ( --YY_G(yy_start_stack_ptr) < 0 )
		YY_FATAL_ERROR( "start-condition stack underflow" );

	BEGIN(YY_G(yy_start_stack)[YY_G(yy_start_stack_ptr)]);
	}
#endif


#ifndef YY_NO_TOP_STATE
/* %- */
#ifdef YY_USE_PROTOS
static int yy_top_state( YY_ONLY_ARG )
#else
static int yy_top_state( YY_ONLY_ARG )
YY_DECL_LAST_ARG
#endif
/* %+ */
/* %* */
	{
	return YY_G(yy_start_stack)[YY_G(yy_start_stack_ptr) - 1];
	}
#endif

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

/* %- */
static void yy_fatal_error( yyconst char msg[] )
	{
	(void) fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
	}

/* %+ */
/* %* */


/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
		yytext[yyleng] = YY_G(yy_hold_char); \
		YY_G(yy_c_buf_p) = yytext + n; \
		YY_G(yy_hold_char) = *YY_G(yy_c_buf_p); \
		*YY_G(yy_c_buf_p) = '\0'; \
		yyleng = n; \
		} \
	while ( 0 )



#ifdef YY_REENTRANT

/* Accessor  methods (get/set functions) to struct members. */

#ifndef YY_NO_GET_EXTRA
#ifdef YY_USE_PROTOS
YY_EXTRA_TYPE yyget_extra( YY_ONLY_ARG )
#else
YY_EXTRA_TYPE yyget_extra( YY_ONLY_ARG )
    YY_DECL_LAST_ARG
#endif
{
    return yyextra;
}
#endif /* !YY_NO_GET_EXTRA */

#ifndef YY_NO_GET_LINENO
#  ifdef YY_USE_PROTOS
int yyget_lineno( YY_ONLY_ARG )
#  else
int yyget_lineno( YY_ONLY_ARG )
    YY_DECL_LAST_ARG
#  endif
{
    return yylineno;
}
#endif /* !YY_NO_GET_LINENO */

#ifndef YY_NO_GET_IN
#ifdef YY_USE_PROTOS
FILE *yyget_in( YY_ONLY_ARG )
#else
FILE *yyget_in( YY_ONLY_ARG )
    YY_DECL_LAST_ARG
#endif
{
    return yyin;
}
#endif /* !YY_NO_GET_IN */

#ifndef YY_NO_GET_OUT
#ifdef YY_USE_PROTOS
FILE *yyget_out( YY_ONLY_ARG )
#else
FILE *yyget_out( YY_ONLY_ARG )
    YY_DECL_LAST_ARG
#endif
{
    return yyout;
}
#endif /* !YY_NO_GET_OUT */

#ifndef YY_NO_GET_LENG
#ifdef YY_USE_PROTOS
int yyget_leng( YY_ONLY_ARG )
#else
int yyget_leng( YY_ONLY_ARG )
    YY_DECL_LAST_ARG
#endif
{
    return yyleng;
}
#endif /* !YY_NO_GET_LENG */

#ifndef YY_NO_GET_TEXT
#ifdef YY_USE_PROTOS
char *yyget_text( YY_ONLY_ARG )
#else
char *yyget_text( YY_ONLY_ARG )
    YY_DECL_LAST_ARG
#endif
{
    return yytext;
}
#endif /* !YY_NO_GET_TEXT */

#ifndef YY_NO_SET_EXTRA
#ifdef YY_USE_PROTOS
void yyset_extra( YY_EXTRA_TYPE user_defined YY_LAST_ARG )
#else
void yyset_extra( user_defined YY_LAST_ARG )
    YY_EXTRA_TYPE user_defined;
    YY_DECL_LAST_ARG
#endif
{
    yyextra = user_defined ;
}
#endif /* !YY_NO_SET_EXTRA */

#ifndef YY_NO_SET_LINENO
#  ifdef YY_USE_PROTOS
void yyset_lineno( int line_number YY_LAST_ARG )
#  else
void yyset_lineno( line_number YY_LAST_ARG )
    int line_number;
    YY_DECL_LAST_ARG
#  endif
{
    yylineno = line_number;
}
#endif /* !YY_NO_SET_LINENO */


#ifndef YY_NO_SET_IN
#ifdef YY_USE_PROTOS
void yyset_in( FILE * in_str YY_LAST_ARG )
#else
void yyset_in( in_str YY_LAST_ARG )
    FILE * in_str;
    YY_DECL_LAST_ARG
#endif
{
    yyin = in_str ;
}
#endif /* !YY_NO_SET_IN */

#ifndef YY_NO_SET_OUT
#ifdef YY_USE_PROTOS
void yyset_out( FILE * out_str YY_LAST_ARG )
#else
void yyset_out( out_str YY_LAST_ARG )
    FILE * out_str;
    YY_DECL_LAST_ARG
#endif
{
    yyout = out_str ;
}
#endif /* !YY_NO_SET_OUT */

/* Accessor methods for yylval and yylloc */

#ifdef YY_REENTRANT_BISON_PURE
#ifndef YY_NO_GET_LVAL
#ifdef YY_USE_PROTOS
YYSTYPE * yyget_lval( YY_ONLY_ARG )
#else
YYSTYPE * yyget_lval( YY_ONLY_ARG )
    YY_DECL_LAST_ARG
#endif
{
    return yylval;
}
#endif /* !YY_NO_GET_LVAL */

#ifndef YY_NO_SET_LVAL
#ifdef YY_USE_PROTOS
void yyset_lval( YYSTYPE * yylvalp YY_LAST_ARG )
#else
void yyset_lval( yylvalp YY_LAST_ARG )
    YYSTYPE * yylvalp;
    YY_DECL_LAST_ARG
#endif
{
    yylval = yylvalp;
}
#endif /* !YY_NO_SET_LVAL */

#ifdef YYLTYPE
#ifndef YY_NO_GET_LLOC
#ifdef YY_USE_PROTOS
YYLTYPE *yyget_lloc( YY_ONLY_ARG )
#else
YYLTYPE *yyget_lloc( YY_ONLY_ARG )
    YY_DECL_LAST_ARG
#endif
{
    return yylloc;
}
#endif /* !YY_NO_GET_LLOC */

#ifndef YY_NO_SET_LLOC
#ifdef YY_USE_PROTOS
void yyset_lloc( YYLTYPE * yyllocp YY_LAST_ARG )
#else
void yyset_lloc( yyllocp YY_LAST_ARG )
    YYLTYPE * yyllocp;
    YY_DECL_LAST_ARG
#endif
{
    yylloc = yyllocp;
}
#endif /* !YY_NO_SET_LLOC */

#endif /* YYLTYPE */
#endif /* YY_REENTRANT_BISON_PURE */


#ifdef YY_USE_PROTOS
static int yy_init_globals( yyscan_t yy_globals)
#else
static int yy_init_globals( yy_globals )
    yyscan_t yy_globals;
#endif
    {
    /* Initialization is the same as for the non-reentrant scanner.
       This function is called once per scanner lifetime. */

  /* We do not touch yylineno unless the option is enabled. */
#ifdef YY_USE_LINENO
    yylineno =  1;
#endif
    YY_G(yy_current_buffer) = 0;
    YY_G(yy_c_buf_p) = (char *) 0;
    YY_G(yy_init) = 1;
    YY_G(yy_start) = 0;
    YY_G(yy_start_stack_ptr) = 0;
    YY_G(yy_start_stack_depth) = 0;
    YY_G(yy_start_stack) = (int *) 0;

/* Defined in main.c */
#ifdef YY_STDINIT
    yyin = stdin;
    yyout = stdout;
#else
    yyin = (FILE *) 0;
    yyout = (FILE *) 0;
#endif
    return 0;
    }

/* User-visible API */
#ifdef YY_USE_PROTOS
int yylex_init( yyscan_t* ptr_yy_globals)
#else
int yylex_init( ptr_yy_globals )
    yyscan_t* ptr_yy_globals;
#endif
    {
    *ptr_yy_globals = (yyscan_t) yy_flex_alloc ( sizeof( struct yy_globals_t ), NULL );
    yy_init_globals ( *ptr_yy_globals );
    return 0;
    }

#ifdef YY_USE_PROTOS
int yylex_destroy( yyscan_t yy_globals )
#else
int yylex_destroy( yy_globals )
    yyscan_t yy_globals;
#endif
    {
    if( yy_globals )
        {

            /* Destroy the current (main) buffer. */
            yy_delete_buffer( YY_G(yy_current_buffer) YY_CALL_LAST_ARG );
            YY_G(yy_current_buffer) = NULL;

            /* Destroy the start condition stack. */
            if( YY_G(yy_start_stack) ) {
                yy_flex_free( YY_G(yy_start_stack) YY_CALL_LAST_ARG );
                YY_G(yy_start_stack) = NULL;
            }

            /* Destroy the main struct. */
            yy_flex_free ( yy_globals YY_CALL_LAST_ARG );
        }
    return 0;
    }

#endif /* End YY_REENTRANT */

/* Internal utility routines. */

#ifndef yytext_ptr
#ifdef YY_USE_PROTOS
static void yy_flex_strncpy( char *s1, yyconst char *s2, int n YY_LAST_ARG)
#else
static void yy_flex_strncpy( s1, s2, n YY_LAST_ARG)
char *s1;
yyconst char *s2;
int n;
YY_DECL_LAST_ARG
#endif
	{
	register int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
	}
#endif

#ifdef YY_NEED_STRLEN
#ifdef YY_USE_PROTOS
static int yy_flex_strlen( yyconst char *s YY_LAST_ARG)
#else
static int yy_flex_strlen( s YY_LAST_ARG)
yyconst char *s;
YY_DECL_LAST_ARG
#endif
	{
	register int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
	}
#endif


#ifdef YY_USE_PROTOS
static void *yy_flex_alloc( yy_size_t size YY_LAST_ARG )
#else
static void *yy_flex_alloc( size YY_LAST_ARG )
yy_size_t size;
YY_DECL_LAST_ARG
#endif
	{
	return (void *) malloc( size );
	}

#ifdef YY_USE_PROTOS
static void *yy_flex_realloc( void *ptr, yy_size_t size YY_LAST_ARG )
#else
static void *yy_flex_realloc( ptr, size YY_LAST_ARG )
void *ptr;
yy_size_t size;
YY_DECL_LAST_ARG
#endif
	{
	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return (void *) realloc( (char *) ptr, size );
	}

#ifdef YY_USE_PROTOS
static void yy_flex_free( void *ptr YY_LAST_ARG )
#else
static void yy_flex_free( ptr YY_LAST_ARG )
void *ptr;
YY_DECL_LAST_ARG
#endif
	{
	free( (char *) ptr );	/* see yy_flex_realloc() for (char *) cast */
	}

#if YY_MAIN
int main()
	{

#ifdef YY_REENTRANT
    yyscan_t lexer;
    yylex_init(&lexer);
    yylex( lexer );
    yylex_destroy( lexer);

#else
	yylex();
#endif

	return 0;
	}
#endif
/* %e */
#endif /* !ssIN_HEADER */
#line 111 "icalsslexer.l"
#ifndef ssIN_HEADER


int yywrap(yyscan_t yy_globals)
{
     return 1;
}

#endif /* !ssIN_HEADER */
